@@ -204,7 +204,7 @@ catalog_list_get_path_from_row (CatalogList *cat_list,
 				gint row)
 {
 	GtkTreePath *path;
-	gchar *result;
+	char        *result;
 
 	g_return_val_if_fail (cat_list != NULL, NULL);
 
@@ -244,7 +244,7 @@ gboolean
 catalog_list_is_catalog (CatalogList *cat_list,
 			 GtkTreeIter *iter)
 {
-	gint type;
+	int type;
 
 	g_return_val_if_fail (cat_list != NULL, FALSE);
 
@@ -260,7 +260,7 @@ gboolean
 catalog_list_is_dir (CatalogList *cat_list,
 		     GtkTreeIter *iter)
 {
-	gint type;
+	int type;
 
 	g_return_val_if_fail (cat_list != NULL, FALSE);
 
@@ -276,7 +276,7 @@ gboolean
 catalog_list_is_search (CatalogList *cat_list,
 			GtkTreeIter *iter)
 {
-	gint type;
+	int type;
 
 	g_return_val_if_fail (cat_list != NULL, FALSE);
 
@@ -333,7 +333,7 @@ catalog_list_update_underline (CatalogList *cat_list)
 
 	g_value_init (&value, PANGO_TYPE_UNDERLINE);
 
-	cat_list->use_underline = pref_get_real_click_policy () == CLICK_POLICY_SINGLE;
+	cat_list->use_underline = pref_get_real_click_policy () == GTH_CLICK_POLICY_SINGLE;
 
 	if (cat_list->use_underline) 
 		g_value_set_enum (&value, PANGO_UNDERLINE_SINGLE);
