@@ -3,7 +3,7 @@
 /*
  *  GThumb
  *
- *  Copyright (C) 2001 The Free Software Foundation, Inc.
+ *  Copyright (C) 2001, 2003 Free Software Foundation, Inc.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -35,8 +35,7 @@
 #include "gthumb-window.h"
 #include "gtk-utils.h"
 #include "gtkcellrendererthreestates.h"
-#include "image-list.h"
-#include "image-list-utils.h"
+#include "gth-file-view.h"
 #include "comments.h"
 
 
@@ -72,9 +71,11 @@ typedef struct {
 
 /* called when the main dialog is closed. */
 static void
-destroy_cb (GtkWidget *widget, 
+destroy_cb (GtkWidget  *widget, 
 	    DialogData *data)
 {
+	data->window->categories_dlg = NULL;
+
 	g_object_unref (data->gui);
 	g_list_foreach (data->file_list, (GFunc) g_free, NULL);
 	g_list_free (data->file_list);
@@ -102,23 +103,20 @@ update_category_entry (DialogData *data)
 		gtk_tree_model_get (model, &iter, USE_CATEGORY_COLUMN, &use_category, -1);
 		if (use_category == 1) {
 			char *utf8_name;
-			char *category_name;
 
 			gtk_tree_model_get (model, &iter, 
 					    CATEGORY_COLUMN, &utf8_name, 
 					    -1);
-			category_name = g_locale_from_utf8 (utf8_name, -1,
-							    NULL, NULL, NULL);
 
 			if (categories->len > 0)
 				categories = g_string_append (categories, CATEGORY_SEPARATOR " ");
-			categories = g_string_append (categories, category_name);
-			g_free (category_name);
+			categories = g_string_append (categories, utf8_name);
+
 			g_free (utf8_name);
 		}
 	} while (gtk_tree_model_iter_next (model, &iter));
 
-	_gtk_entry_set_locale_text (GTK_ENTRY (data->keyword_entry), categories->str);
+	gtk_entry_set_text (GTK_ENTRY (data->keyword_entry), categories->str);
 	g_string_free (categories, TRUE);
 }
 
@@ -144,16 +142,10 @@ get_categories (GtkListStore *store,
 				    USE_CATEGORY_COLUMN, &state, 
 				    CATEGORY_COLUMN, &utf8_name, 
 				    -1);
-		if ((state_to_get == -1) || (state == state_to_get)) {
-			char *name;
-			name = g_locale_from_utf8 (utf8_name, 
-						   -1, 
-						   NULL, 
-						   NULL, 
-						   NULL);
-			list = g_list_prepend (list, name);
-		}
-		g_free (utf8_name);
+		if ((state_to_get == -1) || (state == state_to_get)) 
+			list = g_list_prepend (list, utf8_name);
+		else
+			g_free (utf8_name);
 
 	} while (gtk_tree_model_iter_next (model, &iter));
 
@@ -181,6 +173,16 @@ category_present (DialogData *data,
 }
 
 
+static gboolean
+category_name_is_valid (const char *name)
+{
+	gboolean valid = TRUE;
+	if (strchr (name, ',') != NULL)
+		valid = FALSE;
+	return valid;
+}
+
+
 /* called when the "add category" button is pressed. */
 static void
 add_category_cb (GtkWidget  *widget, 
@@ -188,7 +190,6 @@ add_category_cb (GtkWidget  *widget,
 {
 	GtkTreeIter  iter;
 	char        *new_category;
-	char        *utf8_name;
 
 	new_category = _gtk_request_dialog_run (GTK_WINDOW (data->dialog),
 						GTK_DIALOG_MODAL,
@@ -201,23 +202,26 @@ add_category_cb (GtkWidget  *widget,
 	if (new_category == NULL)
 		return;
 
-	utf8_name = g_locale_to_utf8 (new_category, -1, NULL, NULL, NULL);
-	
-	if (category_present (data, new_category)) {
+	if (! category_name_is_valid (new_category)) {
+		_gtk_error_dialog_run (GTK_WINDOW (data->dialog),
+				       _("The name \"%s\" is not valid because it contains the character \",\". " "Please use a different name."),
+				       new_category);
+
+	} else if (category_present (data, new_category)) {
 		_gtk_error_dialog_run (GTK_WINDOW (data->dialog),
 				       _("The category \"%s\" is already present. Please use a different name."), 
-				       utf8_name);
+				       new_category);
+
 	} else {
 		gtk_list_store_append (data->keywords_list_model, &iter);
 		gtk_list_store_set (data->keywords_list_model, &iter,
 				    IS_EDITABLE_COLUMN, TRUE,
 				    USE_CATEGORY_COLUMN, 0,
-				    CATEGORY_COLUMN, utf8_name,
+				    CATEGORY_COLUMN, new_category,
 				    -1);
 		update_category_entry (data);
 	}
 
-	g_free (utf8_name);
 	g_free (new_category);
 }
 
@@ -251,7 +255,7 @@ save_categories (DialogData *data)
 
 	for (scan = cat_list; scan; scan = scan->next) {
 		char *category = scan->data;
-		bookmarks_add (categories, category, TRUE);
+		bookmarks_add (categories, category, TRUE, TRUE);
 	}
 
 	bookmarks_write_to_disk (categories);
@@ -261,6 +265,16 @@ save_categories (DialogData *data)
 }
 
 
+/* called when the "cancel" button is pressed. */
+static void
+cancel_clicked_cb (GtkWidget  *widget, 
+		   DialogData *data)
+{
+	save_categories (data);
+	gtk_widget_destroy (data->dialog);
+}
+
+
 /* called when the "ok" button is pressed. */
 static void
 ok_clicked_cb (GtkWidget  *widget, 
@@ -356,32 +370,6 @@ use_category_toggled (GtkCellRendererThreeStates *cell,
 }
 
 
-static void
-category_edited (GtkCellRendererText *cell,
-		 gchar               *path_string,
-		 gchar               *new_text,
-		 gpointer             callback_data)
-{
-	DialogData   *data  = callback_data;
-	GtkTreeModel *model = GTK_TREE_MODEL (data->keywords_list_model);
-	GtkTreeIter   iter;
-	GtkTreePath  *path = gtk_tree_path_new_from_string (path_string);
-
-	if (category_present (data, new_text)) {
-		
-		_gtk_error_dialog_run (GTK_WINDOW (data->dialog),
-				       _("The category \"%s\" is already present. Please use a different name."), 
-				       new_text);
-	} else {
-		gtk_tree_model_get_iter (model, &iter, path);
-		gtk_list_store_set (GTK_LIST_STORE (model), &iter, CATEGORY_COLUMN, new_text, -1);
-		update_category_entry (data);
-	}
-
-	gtk_tree_path_free (path);
-}
-
-
 static void
 add_saved_categories (DialogData *data)
 {
@@ -398,7 +386,6 @@ add_saved_categories (DialogData *data)
 		GList       *scan2;
 		gboolean     found = FALSE;
 		char        *category1 = scan->data;
-		char        *utf8_name;
 
 		for (scan2 = cat_list; scan2 && !found; scan2 = scan2->next) {
 			char *category2 = scan2->data;
@@ -411,14 +398,11 @@ add_saved_categories (DialogData *data)
 
 		gtk_list_store_append (data->keywords_list_model, &iter);
 
-		utf8_name = g_locale_to_utf8 (category1,
-                                              -1, NULL, NULL, NULL);
 		gtk_list_store_set (data->keywords_list_model, &iter,
-				    IS_EDITABLE_COLUMN, TRUE,
+				    IS_EDITABLE_COLUMN, FALSE,
 				    USE_CATEGORY_COLUMN, 0,
-				    CATEGORY_COLUMN, utf8_name,
+				    CATEGORY_COLUMN, category1,
 				    -1);
-		g_free (utf8_name);
 	}
 
 	bookmarks_free (categories);
@@ -439,6 +423,27 @@ key_in_list (GList      *list,
 }
 
 
+static int
+name_column_sort_func (GtkTreeModel *model, 
+                       GtkTreeIter  *a, 
+                       GtkTreeIter  *b, 
+                       gpointer      user_data)
+{
+        char *category1, *category2;
+	int   result;
+
+        gtk_tree_model_get (model, a, CATEGORY_COLUMN, &category1, -1);
+        gtk_tree_model_get (model, b, CATEGORY_COLUMN, &category2, -1);
+
+	result = g_utf8_collate (category1, category2);
+
+	g_free (category1);
+	g_free (category2);
+
+        return result;
+}
+
+
 
 void
 dlg_categories (GtkWidget *widget, 
@@ -446,7 +451,6 @@ dlg_categories (GtkWidget *widget,
 {
 	GThumbWindow      *window = wdata;
 	DialogData        *data;
-	ImageList         *ilist;
 	GtkWidget         *btn_ok;
 	GtkWidget         *btn_cancel;
 	GtkWidget         *btn_help;
@@ -457,6 +461,11 @@ dlg_categories (GtkWidget *widget,
 	char              *first_image;
 	GList             *other_keys = NULL;
 
+	if (window->categories_dlg != NULL) {
+		gtk_window_present (GTK_WINDOW (window->categories_dlg));
+		return;
+	}
+
 	data = g_new (DialogData, 1);
 
 	data->window = window;
@@ -467,8 +476,7 @@ dlg_categories (GtkWidget *widget,
                 return;
         }
 
-	ilist = IMAGE_LIST (window->file_list->ilist);
-	data->file_list = ilist_utils_get_file_list_selection (ilist);
+	data->file_list = gth_file_view_get_file_list_selection (window->file_list->view);
 	if (data->file_list == NULL) {
 		g_free (data);
 		return;
@@ -477,6 +485,7 @@ dlg_categories (GtkWidget *widget,
 	/* Get the widgets. */
 
 	data->dialog = glade_xml_get_widget (data->gui, "categories_dialog");
+	window->categories_dlg = data->dialog;
 
 	data->keyword_entry = glade_xml_get_widget (data->gui, "c_keyword_entry");
 	data->add_key_button = glade_xml_get_widget (data->gui, "c_add_key_button");
@@ -517,16 +526,14 @@ dlg_categories (GtkWidget *widget,
 							   "text", CATEGORY_COLUMN,
 							   "editable", IS_EDITABLE_COLUMN,
 							   NULL);
-	g_signal_connect (G_OBJECT (renderer), 
-			  "edited",
-			  G_CALLBACK (category_edited), 
-			  data);
 
 	gtk_tree_view_column_set_sort_column_id (column, 0);
 	gtk_tree_view_append_column (GTK_TREE_VIEW (data->keywords_list_view),
 				     column);
 
-	gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (data->keywords_list_model), CATEGORY_COLUMN, GTK_SORT_ASCENDING);
+	gtk_tree_sortable_set_default_sort_func (GTK_TREE_SORTABLE (data->keywords_list_model), name_column_sort_func, NULL, NULL);
+
+	gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (data->keywords_list_model), GTK_TREE_SORTABLE_DEFAULT_SORT_COLUMN_ID, GTK_SORT_ASCENDING);
 
 	/**/
 
@@ -604,41 +611,32 @@ dlg_categories (GtkWidget *widget,
 
 		for (i = 0; i < cdata->keywords_n; i++) {
 			GtkTreeIter  iter;
-			char        *utf8_name;
 
 			gtk_list_store_append (data->keywords_list_model, 
 					       &iter);
 
-			utf8_name = g_locale_to_utf8 (cdata->keywords[i],
-						      -1, NULL, NULL, NULL);
 			gtk_list_store_set (data->keywords_list_model, &iter,
-					    IS_EDITABLE_COLUMN, TRUE,
+					    IS_EDITABLE_COLUMN, FALSE,
 					    HAS_THIRD_STATE_COLUMN, FALSE,
 					    USE_CATEGORY_COLUMN, 1,
-					    CATEGORY_COLUMN, utf8_name,
+					    CATEGORY_COLUMN, cdata->keywords[i],
 					    -1);
-			g_free (utf8_name);
 		}
 	}
 
 	for (scan = other_keys; scan; scan = scan->next) {
 		char        *keyword = scan->data;
 		GtkTreeIter  iter;
-		char        *utf8_name;
 		
 		gtk_list_store_append (data->keywords_list_model, 
 				       &iter);
 		
-		utf8_name = g_locale_to_utf8 (keyword,
-					      -1, NULL, NULL, NULL);
-		
 		gtk_list_store_set (data->keywords_list_model, &iter,
-				    IS_EDITABLE_COLUMN, TRUE,
+				    IS_EDITABLE_COLUMN, FALSE,
 				    HAS_THIRD_STATE_COLUMN, TRUE,
 				    USE_CATEGORY_COLUMN, 2,
-				    CATEGORY_COLUMN, utf8_name,
+				    CATEGORY_COLUMN, keyword,
 				    -1);
-		g_free (utf8_name);
 	}
 
 	if (other_keys != NULL)
@@ -657,10 +655,10 @@ dlg_categories (GtkWidget *widget,
 			  "clicked",
 			  G_CALLBACK (ok_clicked_cb),
 			  data);
-	g_signal_connect_swapped (G_OBJECT (btn_cancel), 
-				  "clicked",
-				  G_CALLBACK (gtk_widget_destroy),
-				  G_OBJECT (data->dialog));
+	g_signal_connect (G_OBJECT (btn_cancel), 
+			  "clicked",
+			  G_CALLBACK (cancel_clicked_cb),
+			  data);
 	g_signal_connect (G_OBJECT (btn_help), 
 			  "clicked",
 			  G_CALLBACK (help_cb),
@@ -679,7 +677,7 @@ dlg_categories (GtkWidget *widget,
 
 	gtk_window_set_transient_for (GTK_WINDOW (data->dialog), 
 				      GTK_WINDOW (window->app));
-	gtk_window_set_modal (GTK_WINDOW (data->dialog), TRUE);
+	gtk_window_set_modal (GTK_WINDOW (data->dialog), FALSE);
 	gtk_widget_show (data->dialog);
 
 	gtk_widget_grab_focus (data->keywords_list_view);
