@@ -3,7 +3,7 @@
 /*
  *  GThumb
  *
- *  Copyright (C) 2001 The Free Software Foundation, Inc.
+ *  Copyright (C) 2001, 2003 Free Software Foundation, Inc.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -25,8 +25,17 @@
 #include <string.h>
 #include <unistd.h>
 #include <gtk/gtk.h>
+#include <libgnome/libgnome.h>
 #include <libgnomevfs/gnome-vfs-mime.h>
 #include <glade/glade.h>
+
+#ifdef HAVE_LIBEXIF
+#include <exif-data.h>
+#include <exif-content.h>
+#include <exif-entry.h>
+#include "jpegutils/jpeg-data.h"
+#endif /* HAVE_LIBEXIF */
+
 #include "file-data.h"
 #include "gconf-utils.h"
 #include "gthumb-window.h"
@@ -58,6 +67,7 @@ typedef struct {
 	GladeXML     *gui;
 
 	GtkWidget    *dialog;
+	GtkWidget    *j_button_box;
 	GtkWidget    *j_apply_to_all_checkbutton;
 
 	int           rot_type;    /* 0, 90 ,180, 270 */
@@ -73,9 +83,8 @@ static void
 destroy_cb (GtkWidget  *widget, 
 	    DialogData *data)
 {
-	if (data->file_list != NULL) 
-		g_list_free (data->file_list);
-	g_object_unref (G_OBJECT (data->gui));
+	file_data_list_free (data->file_list);
+	g_object_unref (data->gui);
 	g_free (data);
 }
 
@@ -94,6 +103,14 @@ add_image_to_button (GtkWidget    *button,
 }
 
 
+static void
+image_loaded_cb (ImageViewer *viewer,
+		 DialogData  *data)
+{
+	gtk_widget_set_sensitive (data->j_button_box, TRUE);
+}
+
+
 static void
 load_current_image (DialogData *data)
 {
@@ -104,6 +121,8 @@ load_current_image (DialogData *data)
 		return;
 	}
 	
+	gtk_widget_set_sensitive (data->j_button_box, FALSE);
+
 	fd = data->current_image->data;
 	image_viewer_load_image (IMAGE_VIEWER (data->viewer), fd->path);
 
@@ -130,6 +149,90 @@ load_next_image (DialogData *data)
 }
 
 
+#ifdef HAVE_LIBEXIF
+
+
+static gboolean
+swap_fields (ExifContent *content,
+	     ExifTag      tag1,
+	     ExifTag      tag2)
+{
+	ExifEntry     *entry1;
+	ExifEntry     *entry2;
+	unsigned char *data;
+	unsigned int   size;
+	
+	entry1 = exif_content_get_entry (content, tag1);
+	if (entry1 == NULL) 
+		return FALSE;
+	
+	entry2 = exif_content_get_entry (content, tag2);
+	if (entry2 == NULL)
+		return FALSE;
+
+	data = entry1->data;
+	size = entry1->size;
+	
+	entry1->data = entry2->data;
+	entry1->size = entry2->size;
+	
+	entry2->data = data;
+	entry2->size = size;
+
+	return TRUE;
+}
+
+
+static void
+swap_xy_exif_fields (const char *filename)
+{
+	JPEGData     *jdata;
+	ExifData     *edata;
+	unsigned int  i;
+
+	jdata = jpeg_data_new_from_file (filename);
+	if (jdata == NULL)
+		return;
+
+	edata = jpeg_data_get_exif_data (jdata);
+	if (edata == NULL) {
+		jpeg_data_unref (jdata);
+		return;
+	}
+
+	for (i = 0; i < EXIF_IFD_COUNT; i++) {
+		ExifContent *content = edata->ifd[i];
+
+		if ((content == NULL) || (content->count == 0)) 
+			continue;
+		
+		swap_fields (content, 
+			     EXIF_TAG_RELATED_IMAGE_WIDTH,
+			     EXIF_TAG_RELATED_IMAGE_LENGTH);
+		swap_fields (content, 
+			     EXIF_TAG_IMAGE_WIDTH,
+			     EXIF_TAG_IMAGE_LENGTH);
+		swap_fields (content, 
+			     EXIF_TAG_PIXEL_X_DIMENSION,
+			     EXIF_TAG_PIXEL_Y_DIMENSION);
+		swap_fields (content, 
+			     EXIF_TAG_X_RESOLUTION,
+			     EXIF_TAG_Y_RESOLUTION);
+		swap_fields (content, 
+			     EXIF_TAG_FOCAL_PLANE_X_RESOLUTION,
+			     EXIF_TAG_FOCAL_PLANE_Y_RESOLUTION);
+	}
+
+	jpeg_data_save_file (jdata, filename);
+
+	exif_data_unref (edata);
+	jpeg_data_unref (jdata);
+}
+
+
+#endif
+
+
 static void
 apply_tran (DialogData *data,
 	    GList      *current_image)
@@ -176,11 +279,10 @@ apply_tran (DialogData *data,
 			break;
 		}
 
-		if (jpegtran (fd->path, tmp1, transf) != 0) {
+		if (jpegtran (fd->path, tmp1, transf, &err) != 0) {
 			g_free (tmp1);
-			
-			/* FIXME : error dialog here */
-
+			if (err != NULL) 
+				_gtk_error_dialog_from_gerror_run (parent, &err);
 			return;
 		}
 
@@ -203,14 +305,11 @@ apply_tran (DialogData *data,
 		e2 = shell_escape (fd->path);
 	
 		line = g_strdup_printf ("jpegtran -copy all %s -outfile %s %s",
-					command, 
-					e1, 
-					e2);
+					command, e1, e2);
+		g_spawn_command_line_sync (line, NULL, NULL, NULL, &err);  
 
 		g_free (e1);
 		g_free (e2);
-
-		g_spawn_command_line_sync (line, NULL, NULL, NULL, &err);  
 		g_free (line);
 
 		if (err != NULL) {
@@ -242,11 +341,10 @@ apply_tran (DialogData *data,
 			break;
 		}
 
-		if (jpegtran (tmp1, tmp2, transf) != 0) {
+		if (jpegtran (tmp1, tmp2, transf, &err) != 0) {
 			g_free (tmp1);
-			
-			/* FIXME : error dialog here */
-
+			if (err != NULL) 
+				_gtk_error_dialog_from_gerror_run (parent, &err);
 			return;
 		}
 
@@ -266,14 +364,11 @@ apply_tran (DialogData *data,
 		e2 = shell_escape (tmp1);
 
 		line = g_strdup_printf ("jpegtran -copy all %s -outfile %s %s",
-					command, 
-					e1, 
-					e2);
+					command, e1, e2);
+		g_spawn_command_line_sync (line, NULL, NULL, NULL, &err);  
 
 		g_free (e1);
 		g_free (e2);
-
-		g_spawn_command_line_sync (line, NULL, NULL, NULL, &err);  
 		g_free (line);
 
 		if (err != NULL) {
@@ -289,25 +384,29 @@ apply_tran (DialogData *data,
 	e2 = shell_escape (fd->path);
 
 	line = g_strdup_printf ("mv -f %s %s", e1, e2);
-
-	g_free (e1);
-	g_free (e2);
-
 	g_spawn_command_line_sync (line, NULL, NULL, NULL, &err);  
-	g_free (line);
-
-	g_free (tmp1);
-	g_free (tmp2);
 
 	if (err != NULL) {
 		_gtk_error_dialog_from_gerror_run (parent, &err);
+
 	} else {
 		GList *list;
 
+#ifdef HAVE_LIBEXIF
+		if ((rot_type == TRAN_ROTATE_90) || (rot_type == TRAN_ROTATE_270))
+			swap_xy_exif_fields (fd->path);
+#endif
+
 		list = g_list_prepend (NULL, fd->path);
 		all_windows_notify_files_changed (list);
 		g_list_free (list);
 	}
+
+	g_free (e1);
+	g_free (e2);
+	g_free (line);
+	g_free (tmp1);
+	g_free (tmp2);
 }
 
 
@@ -327,12 +426,14 @@ ok_clicked (GtkWidget  *button,
 		GList     *scan;
 		int        i, n;
 
+		gtk_widget_hide (data->dialog);
+
 		gui = glade_xml_new (GTHUMB_GLADEDIR "/" PROGRESS_GLADE_FILE, 
 				     NULL,
 				     NULL);
 
-		dialog = glade_xml_get_widget   (gui, "progress_dialog");
-		label = glade_xml_get_widget    (gui, "progress_info");
+		dialog = glade_xml_get_widget (gui, "progress_dialog");
+		label = glade_xml_get_widget (gui, "progress_info");
 		bar = glade_xml_get_widget (gui, "progress_progressbar");
 
 		n = g_list_length (data->current_image);
@@ -406,9 +507,15 @@ rot90_clicked (GtkWidget  *button,
 	}
 
 	src_pixbuf = image_viewer_get_current_pixbuf (viewer);
+
+	if (src_pixbuf == NULL) 
+		return;
+
 	dest_pixbuf = _gdk_pixbuf_copy_rotate_90 (src_pixbuf, FALSE);
 	image_viewer_set_pixbuf (viewer, dest_pixbuf);
-	g_object_unref (dest_pixbuf);
+
+	if (dest_pixbuf != NULL) 
+		g_object_unref (dest_pixbuf);
 }
 
 
@@ -428,9 +535,15 @@ rot270_clicked (GtkWidget  *button,
 		data->rot_type = get_next_rot (data->rot_type);
 
 	src_pixbuf = image_viewer_get_current_pixbuf (viewer);
+
+	if (src_pixbuf == NULL) 
+		return;
+
 	dest_pixbuf = _gdk_pixbuf_copy_rotate_90 (src_pixbuf, TRUE);
 	image_viewer_set_pixbuf (viewer, dest_pixbuf);
-	g_object_unref (dest_pixbuf);
+
+	if (dest_pixbuf != NULL) 
+		g_object_unref (dest_pixbuf);
 }
 
 
@@ -438,23 +551,31 @@ static void
 mirror_clicked (GtkWidget  *button, 
 		DialogData *data)
 {
-	ImageViewer  *viewer = IMAGE_VIEWER (data->viewer);
-	GdkPixbuf    *src_pixbuf;
-	GdkPixbuf    *dest_pixbuf;
+	ImageViewer *viewer = IMAGE_VIEWER (data->viewer);
+	GdkPixbuf   *src_pixbuf;
+	GdkPixbuf   *dest_pixbuf;
+
+	src_pixbuf = image_viewer_get_current_pixbuf (viewer);
+
+	if (src_pixbuf == NULL) 
+		return;
 
 	if (data->tran_type == TRAN_FLIP) {
 		data->tran_type = TRAN_NONE;
 		data->rot_type = get_next_rot (data->rot_type);
 		data->rot_type = get_next_rot (data->rot_type);
+
 	} else if (data->tran_type == TRAN_MIRROR)
 		data->tran_type = TRAN_NONE;
+
 	else
 		data->tran_type = TRAN_MIRROR;
-
-	src_pixbuf = image_viewer_get_current_pixbuf (viewer);
+	
 	dest_pixbuf = _gdk_pixbuf_copy_mirror (src_pixbuf, TRUE, FALSE);
 	image_viewer_set_pixbuf (viewer, dest_pixbuf);
-	g_object_unref (dest_pixbuf);
+	
+	if (dest_pixbuf != NULL) 
+		g_object_unref (dest_pixbuf);
 }
 
 
@@ -470,15 +591,56 @@ flip_clicked (GtkWidget  *button,
 		data->tran_type = TRAN_NONE;
 		data->rot_type = get_next_rot (data->rot_type);
 		data->rot_type = get_next_rot (data->rot_type);
+
 	} else if (data->tran_type == TRAN_FLIP)
 		data->tran_type = TRAN_NONE;
+
 	else
 		data->tran_type = TRAN_FLIP;
 
 	src_pixbuf = image_viewer_get_current_pixbuf (viewer);
+
+	if (src_pixbuf == NULL) 
+		return;
+
 	dest_pixbuf = _gdk_pixbuf_copy_mirror (src_pixbuf, FALSE, TRUE);
 	image_viewer_set_pixbuf (viewer, dest_pixbuf);
-	g_object_unref (dest_pixbuf);
+
+	if (dest_pixbuf != NULL) 
+		g_object_unref (dest_pixbuf);
+}
+
+
+/* called when the "help" button is clicked. */
+static void
+help_cb (GtkWidget  *widget, 
+	 DialogData *data)
+{
+	GError *err;
+
+	err = NULL;  
+	gnome_help_display ("gthumb", "rotate-jpeg", &err);
+	
+	if (err != NULL) {
+		GtkWidget *dialog;
+		
+		dialog = gtk_message_dialog_new (GTK_WINDOW (data->dialog),
+						 0,
+						 GTK_MESSAGE_ERROR,
+						 GTK_BUTTONS_CLOSE,
+						 _("Could not display help: %s"),
+						 err->message);
+		
+		g_signal_connect (G_OBJECT (dialog), "response",
+				  G_CALLBACK (gtk_widget_destroy),
+				  NULL);
+		
+		gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
+		
+		gtk_widget_show (dialog);
+		
+		g_error_free (err);
+	}
 }
 
 
@@ -497,7 +659,7 @@ dlg_jpegtran (GThumbWindow *window)
 	GtkWidget   *j_ok_button;
 	GList       *list, *scan;
 
-	list = file_list_get_selection_as_fd (window->file_list);
+	list = gth_file_list_get_selection_as_fd (window->file_list);
 	if (list == NULL) {
 		g_warning ("No file selected.");
 		return;
@@ -512,6 +674,7 @@ dlg_jpegtran (GThumbWindow *window)
 		if (! image_is_jpeg (fd->path)) {
 			list = g_list_remove_link (list, scan);
 			g_list_free (scan);
+			file_data_unref (fd);
 		}
 
 		scan = next;
@@ -543,6 +706,7 @@ dlg_jpegtran (GThumbWindow *window)
 
 	data->dialog = glade_xml_get_widget (data->gui, "jpeg_rotate_dialog");
 	data->j_apply_to_all_checkbutton = glade_xml_get_widget (data->gui, "j_apply_to_all_checkbutton");
+	data->j_button_box = glade_xml_get_widget (data->gui, "j_button_box");
 
 	j_image_vbox = glade_xml_get_widget (data->gui, "j_image_vbox");
 	j_revert_button = glade_xml_get_widget (data->gui, "j_revert_button");
@@ -550,6 +714,7 @@ dlg_jpegtran (GThumbWindow *window)
 	j_rot_270_button = glade_xml_get_widget (data->gui, "j_rot_270_button");
 	j_v_flip_button = glade_xml_get_widget (data->gui, "j_v_flip_button");
 	j_h_flip_button = glade_xml_get_widget (data->gui, "j_h_flip_button");
+
 	j_help_button = glade_xml_get_widget (data->gui, "j_help_button");
 	j_cancel_button = glade_xml_get_widget (data->gui, "j_cancel_button");
 	j_ok_button = glade_xml_get_widget (data->gui, "j_ok_button");
@@ -559,7 +724,7 @@ dlg_jpegtran (GThumbWindow *window)
 	data->viewer = image_viewer_new ();
 	image_viewer_size (IMAGE_VIEWER (data->viewer), PREVIEW_SIZE, PREVIEW_SIZE);
 	gtk_container_add (GTK_CONTAINER (j_image_vbox), data->viewer);
-	image_viewer_set_zoom_change (IMAGE_VIEWER (data->viewer), ZOOM_CHANGE_FIT_IF_LARGER);
+	image_viewer_set_zoom_change (IMAGE_VIEWER (data->viewer), GTH_ZOOM_CHANGE_FIT_IF_LARGER);
 	image_viewer_set_zoom_quality (IMAGE_VIEWER (data->viewer),
 				       pref_get_zoom_quality ());
 	image_viewer_set_check_type (IMAGE_VIEWER (data->viewer),
@@ -590,6 +755,10 @@ dlg_jpegtran (GThumbWindow *window)
 				  "clicked",
 				  G_CALLBACK (gtk_widget_destroy),
 				  G_OBJECT (data->dialog));
+	g_signal_connect (G_OBJECT (j_help_button), 
+			  "clicked",
+			  G_CALLBACK (help_cb),
+			  data);
 	g_signal_connect (G_OBJECT (j_ok_button), 
 			  "clicked",
 			  G_CALLBACK (ok_clicked),
@@ -615,6 +784,11 @@ dlg_jpegtran (GThumbWindow *window)
 			  G_CALLBACK (flip_clicked),
 			  data);
 
+	g_signal_connect (G_OBJECT (data->viewer),
+			  "image_loaded",
+			  G_CALLBACK (image_loaded_cb),
+			  data);
+
 	/* Run dialog. */
 
 	gtk_window_set_transient_for (GTK_WINDOW (data->dialog), GTK_WINDOW (window->app));
